indexing

   description: "Class that covers a path on the file system.",
   "Also makes possible to use either a / or \ as directory separator",
   "(Unix or Windows convention).",
   "Secondly, it does environment variable expansion for names as",
   "$(TMP), ${TMP}, $TMP or %%TMP%% (read single percent sign here!).",
   "Also recognizes ~ and replaces by $HOME except if ~ is followed by a",
   "digit, so it should work more or less on Windows when it should, and",
   "do nothing when it shouldn't."

   author: "Berend de Boer"
   date: "$Date: Fri, 13 Oct 2000 19:01:32 +0200 $";
   revision: "$Revision: 1.5.1.2.1.2 $"

class 

   STDC_PATH


inherit
   
   STRING
      rename
         make as string_make
      end


creation
   
   make,
   string_make

   
feature

   make, make_expand (s: STRING) is
         -- the new path is the expanded contents od `s'
      do
         make_from_string (normalize_path (s))
      end


feature -- other

   is_portable_character (c: CHARACTER): BOOLEAN is
         -- return True if c is a POSIX portable character, i.e. it 
         -- can safely be used in filenames
      do
         Result := 
            (c >= 'a' and c <= 'z') or else
            (c = '.') or else
            (c = '_') or else
            (c >= '0' and c <= '9') or else
            (c >= 'A' and c <= 'Z') or else
            (c = '-');
      end


feature {NONE}
   
   normalize_path (s: STRING): STRING is
      local
         i, j: INTEGER
         start_name, stop_name: INTEGER
         c: CHARACTER
         env_name: STRING
      do
         create Result.make (s.count * 2)
         from
            i := 1
         until
            i > s.count
         loop
            c := s.item (i)
            inspect c
            when '~' then
               if (i = s.count) or else not is_digit (s.item (i+1)) then
                  Result.append_string (home_directory)
               else
                  Result.append_character (c)
               end
            when '\' then
               Result.append_character (directory_separator)
            when '/' then
               Result.append_character (directory_separator)
            when '$' then
               if (i < s.count) then
                  inspect s.item (i+1)
                  when '{' then
                     start_name := i+2
                     j := my_index_of (s, '}', start_name)
                     stop_name := j-1
                     i := j
                  when '(' then
                     start_name := i+2
                     j := my_index_of (s, ')', start_name)
                     stop_name := j-1
                     i := j
                  when '$' then
                     -- on Windows you can have temp names like tmp1.$$$
                     Result.append_character (c)
                  else
                     start_name := i+1
                     from
                        j := start_name
                     until
                        j > s.count or else not is_env_character (s.item(j))
                     loop                        
                        j := j + 1
                     end
                     stop_name := j-1
                     i := j-1
                  end
                  if start_name <= stop_name - 1 then
                     env_name := s.substring (start_name, stop_name)
                     Result.append_string (env_value (env_name))
                  end
               else
                  -- if last character is a $, do not consider it a
                  -- special character
                  Result.append_character (c)
               end
            when '%%' then
               start_name := i+1
               j := my_index_of (s, '%%', start_name)
               stop_name := j-1
               i := j
               env_name := s.substring (start_name, stop_name)
               Result.append_string (env_value (env_name))
            else
               Result.append_character (c)
            end
            i := i + 1
         end
      end

   directory_separator: CHARACTER is
         -- attempts to return a platform specific directory separator
      once
         if is_windows then
            Result := '\'
         else
            Result := '/'
         end
      end

   env_value (env_name: STRING): STRING is
         -- return value of environment variable `env_name'
      local
         e: STDC_ENV_VAR
      do
         create e.make (env_name)
         Result := e.value
      end

   home_directory: STRING is
      local
         env_home: STDC_ENV_VAR
      do
         create env_home.make ("HOME")
         Result := env_home.value
      end
   
   is_digit (c: CHARACTER): BOOLEAN is
         -- return True if c is a digit, it seems with Eiffel you can 
         -- write every shitty routine yourself it you want compatibility
      do
         Result := (c >= '0') and (c <= '9');
      end
   
   is_env_character (c: CHARACTER): BOOLEAN is
         -- return True if c can be used in an environment variable 
         -- used by our expansion routine.
      do
         Result := 
            (c >= 'A' and c <= 'Z') or else
            (c >= 'a' and c <= 'z') or else
            (c >= '0' and c <= '9');
      end

   my_index_of (s: STRING; c: CHARACTER; start: INTEGER): INTEGER is
      do
         from
            Result := start
         until
            Result > s.count or else s.item (Result) = c
         loop
            Result := Result + 1
         end
      end
      

feature {NONE}

   is_windows: BOOLEAN is
      external "C"
      end


end -- class STDC_PATH
